<?php
/*
Plugin Name: LearnDash LMS - 2Checkout Integration
Plugin URI: http://www.learndash.com
Description: 2Checkout payment gateway for LearnDash LMS. 
Version: 1.1.1.1
Author: LearnDash
Author URI: http://www.learndash.com
Text Domain: learndash-2checkout
Doman Path: /languages/
*/

// Plugin version
if ( ! defined( 'LEARNDASH_2CHECKOUT_VERSION' ) ) {
	define( 'LEARNDASH_2CHECKOUT_VERSION', '1.1.1.1' );
}

// Plugin file
if ( ! defined( 'LEARNDASH_2CHECKOUT_FILE' ) ) {
	define( 'LEARNDASH_2CHECKOUT_FILE', __FILE__ );
}		

// Plugin folder path
if ( ! defined( 'LEARNDASH_2CHECKOUT_PLUGIN_PATH' ) ) {
	define( 'LEARNDASH_2CHECKOUT_PLUGIN_PATH', plugin_dir_path( __FILE__ ) );
}

// Plugin folder URL
if ( ! defined( 'LEARNDASH_2CHECKOUT_PLUGIN_URL' ) ) {
	define( 'LEARNDASH_2CHECKOUT_PLUGIN_URL', plugin_dir_url( __FILE__ ) );
}

/**
 * Check plugin dependencies
 * @return void
 */
function learndash_2checkout_check_dependency()
{
	include LEARNDASH_2CHECKOUT_PLUGIN_PATH . 'includes/class-dependency-check.php';

	LearnDash_Dependency_Check_LD_2Checkout::get_instance()->set_dependencies(
		array(
			'sfwd-lms/sfwd_lms.php' => array(
				'label'       => '<a href="https://learndash.com">LearnDash LMS</a>',
				'class'       => 'SFWD_LMS',
				'min_version' => '3.0.0',
			),
		)
	);

	LearnDash_Dependency_Check_LD_2Checkout::get_instance()->set_message(
		__( 'LearnDash LMS - 2Checkout Add-on requires the following plugin(s) to be active:', 'learndash-2checkout' )
	);
}

learndash_2checkout_check_dependency();
add_action( 'plugins_loaded', 'learndash_2checkout_init' );

function learndash_2checkout_init() {
	if ( LearnDash_Dependency_Check_LD_2Checkout::get_instance()->check_dependency_results() ) {
		include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
		include_once( plugin_dir_path( __FILE__ ) . 'checkout_process.php' );

		add_action( 'admin_menu', 'learndash_payment_button_2checkout_menu', 1100 );
		add_filter( 'learndash_admin_tabs', 'learndash_2checkout_admin_tabs', 1, 1 );
		add_filter( 'learndash_admin_tabs_on_page', 'learndash_2checkout_learndash_admin_tabs_on_page', 3, 3 );

		add_action( 'plugins_loaded', 'learndash_2checkout_i18ize' );

		add_filter( 'learndash_payment_button', 'learndash_payment_button_2checkout', 1, 2 );

		add_action( 'wp_footer', 'learndash_2checkout_output_message' );
	}
}

function learndash_2checkout_i18ize() {
	load_plugin_textdomain( 'learndash_2checkout', false, dirname( plugin_basename( __FILE__ ) ) . '/languages/' ); 

	// include translation/update class
	include LEARNDASH_2CHECKOUT_PLUGIN_PATH . 'includes/class-translations-ld-2checkout.php';
}

function learndash_payment_button_2checkout_menu() {
	add_submenu_page( 'edit.php?post_type=sfwd-courses', __( '2Checkout Settings','learndash_2checkout' ), __( '2Checkout Settings','learndash_2checkout' ), 'manage_options','admin.php?page=learndash-2checkout-settings', 'learndash_payment_button_2checkout_menupage' );
	add_submenu_page( 'learndash-lms-non-existant', __( '2Checkout Settings', 'learndash_2checkout' ), __( '2Checkout Settings', 'learndash_2checkout' ), 'manage_options', 'learndash-2checkout-settings', 'learndash_payment_button_2checkout_menupage' );
}

function learndash_2checkout_admin_tabs( $admin_tabs ) {
	$admin_tabs['2checkout'] = array(
		'link'	=> 'admin.php?page=learndash-2checkout-settings',
		'name'	=> __( '2Checkout Settings','learndash_2checkout' ),
		'id'	=> 'admin_page_learndash-2checkout-settings',
		'menu_link'	=> 'edit.php?post_type=sfwd-courses&page=sfwd-lms_sfwd_lms.php_post_type_sfwd-courses',
	);

	return $admin_tabs;
}

function learndash_2checkout_learndash_admin_tabs_on_page( $admin_tabs_on_page, $admin_tabs, $current_page_id ) {
	if ( isset( $admin_tabs_on_page['admin_page_learndash-2checkout-settings'] ) ) {
		$admin_tabs_on_page['admin_page_learndash-2checkout-settings'] = array_merge( $admin_tabs_on_page['sfwd-courses_page_sfwd-lms_sfwd_lms_post_type_sfwd-courses'], ( array ) $admin_tabs_on_page['admin_page_learndash-2checkout-settings'] );
	} else {
		$admin_tabs_on_page['admin_page_learndash-2checkout-settings'] = array_merge( $admin_tabs_on_page['sfwd-courses_page_sfwd-lms_sfwd_lms_post_type_sfwd-courses'], array() );
	}

	foreach ( $admin_tabs as $key => $value ) {
		if ( $value['id'] == $current_page_id && $value['menu_link'] == 'edit.php?post_type=sfwd-courses&page=sfwd-lms_sfwd_lms.php_post_type_sfwd-courses' )
		{
			$admin_tabs_on_page[$current_page_id][] = '2checkout';
			return $admin_tabs_on_page;
		}
	}
	return $admin_tabs_on_page;
}

function learndash_payment_button_2checkout_menupage() {
   	// Must check that the user has the required capability 
    if ( ! current_user_can( 'manage_options' ) )
    {
      wp_die( __( 'You do not have sufficient permissions to access this page.' ) );
    }
	
	$learndash_2checkout_settings = get_option( 'learndash_2checkout_settings' );
	$learndash_2checkout_settings_sid = ! empty( $learndash_2checkout_settings['sid'] ) ? $learndash_2checkout_settings['sid'] : '';
	$learndash_2checkout_settings_secret_word = ! empty( $learndash_2checkout_settings['secret_word'] ) ? $learndash_2checkout_settings['secret_word'] : '';
	$learndash_2checkout_settings_demo = ! empty( $learndash_2checkout_settings['demo'] ) ? $learndash_2checkout_settings['demo'] : '';
	$x_receipt_link_url = site_url( '/' ) . '?learndash-checkout=2co';
	
	
	if ( isset( $_POST[ 'update_LearnDash2CheckoutSettings' ] ) ) {
		$learndash_2checkout_settings_sid = $_POST['learndash_2checkout_settings_sid'];
		$learndash_2checkout_settings_secret_word = $_POST['learndash_2checkout_settings_secret_word'];
		$learndash_2checkout_settings_demo = empty( $_POST['learndash_2checkout_settings_demo'] ) ? 0 : 1;
		$learndash_2checkout_settings['sid'] = $learndash_2checkout_settings_sid;
		$learndash_2checkout_settings['secret_word'] = $learndash_2checkout_settings_secret_word;
		$learndash_2checkout_settings['demo'] = $learndash_2checkout_settings_demo;
	
		update_option( 'learndash_2checkout_settings', $learndash_2checkout_settings );
		?>
		<div class="updated"><p><strong><?php _e( 'settings saved.', 'learndash_2checkout' ); ?></strong></p></div>
		<?php
	}
	
	
	?>
	<div class="wrap">
	<form method="post" action="<?php echo esc_url( $_SERVER['REQUEST_URI'] ); ?>">
	<h2><?php _e( 'LearnDash - 2Checkout Payment Gateway Settings', 'learndash_2checkout' ); ?></h2>
	<br>
	<h3><?php _e( 'Merchant Code:', 'learndash_2checkout' ); ?></h3>
	<input name="learndash_2checkout_settings_sid" style="min-width: 30%" value="<?php esc_attr_e( apply_filters( 'format_to_edit', $learndash_2checkout_settings_sid ), 'learndash_2checkout' ) ?>" />
	<p class="description"><?php _e( 'Your 2Checkout merchant code/account number.', 'learndash_2checkout' ); ?></p>
	<br>
	<h3><?php _e( 'Buy Link Secret Word:', 'learndash_2checkout' ); ?></h3>
	<input name="learndash_2checkout_settings_secret_word" style="min-width: 30%" value="<?php esc_attr_e( apply_filters( 'format_to_edit', $learndash_2checkout_settings_secret_word ), 'learndash_2checkout' ) ?>" />
	<p class="description"><?php _e( 'Must match with the Buy Link secret word in your 2Checkout account.', 'learndash_2checkout' ); ?></p>
	<br>
	<h3><?php _e( 'Approved URL ( Header Redirect ):', 'learndash_2checkout' ) ?></h3>
	<input type="text" style="min-width: 30%" value="<?php echo esc_url( $x_receipt_link_url ); ?>" readonly="readonly">
	<p class="description"><?php _e( 'Enter this URL into approved URL setting in your 2Checkout account with "header redirect" return method.', 'learndash_2checkout' ); ?></p>
	<br>
	<h3><input name="learndash_2checkout_settings_demo" type="checkbox" style="min-width: 0%" <?php if ( ! empty( $learndash_2checkout_settings_demo ) ) echo 'checked="checked"'; ?> /> <?php _e( 'Demo Mode', 'learndash_2checkout' ); ?></h3>
	<p class="description"><?php _e( 'Check to enable demo mode.', 'learndash_2checkout' ); ?></p>
	<div class="submit">
	<input type="submit" name="update_LearnDash2CheckoutSettings" value="<?php _e( 'Update Settings', 'learndash_2checkout' ) ?>" /></div>
	</form>
	<?php
}

function learndash_payment_button_2checkout( $button, $params = null ) {
	$learndash_2checkout_settings = get_option( 'learndash_2checkout_settings' );
	$sid = ! empty( $learndash_2checkout_settings['sid'] ) ? $learndash_2checkout_settings['sid'] : '';
	$demo = ! empty( $learndash_2checkout_settings['demo'] ) ? 'Y' : 'N';
	
	if ( empty( $learndash_2checkout_settings ) || empty( $sid ) || empty( $params ) || empty( $params['price'] ) || empty( $params['post'] ) || empty( $params['post']->ID ) )
		return $button;

	$price = number_format( $params['price'], 2, '.', '' );
	$post =  $params['post'];
	$title = $post->post_title;
	$productid = $post->ID;
	$order_id = time() . '-' . rand( 100, 999 );

	$action_url = 'https://www.2checkout.com/checkout/purchase';
	$x_receipt_link_url = site_url( '/' ) . '?learndash-checkout=2co';
	
	$current_user = wp_get_current_user();
	$user_id = empty( $current_user->ID ) ? 0 : $current_user->ID;
	$price_type = learndash_get_setting( $productid, 'course_price_type' );
	$recurring_cycle_number = get_post_meta( $productid, 'course_price_billing_p3', true );
	$recurring_cycle = get_post_meta( $productid, 'course_price_billing_t3', true );

	switch ( $recurring_cycle ) {
		case 'D':
			$recurring_cycle_period = 'Day';
			break;
		
		case 'W':
			$recurring_cycle_period = 'Week';
			break;

		case 'M':
			$recurring_cycle_period = 'Month';
			break;

		case 'Y':
			$recurring_cycle_period = 'Year';
			break;

		default:
			$recurring_cycle_period = 'Day';
			break;
	}

	$recurrence = $price_type == 'subscribe' ? '<input type="hidden" name="li_0_recurrence" value="' . esc_attr( $recurring_cycle_number . ' ' . $recurring_cycle_period ) . '">' : '';

	if ( strstr( $button, 'learndash_checkout_button' ) ) {
		$button_text  = apply_filters( 'learndash_2checkout_purchase_button_text', __( 'Use 2Checkout', 'learndash-2checkout' ) );
	} else {
	    if ( class_exists( 'LearnDash_Custom_Label' ) ) {
	        $button_text  = apply_filters( 'learndash_2checkout_purchase_button_text', LearnDash_Custom_Label::get_label( 'button_take_this_course' ) );
	    } else {
	        $button_text  = apply_filters( 'learndash_2checkout_purchase_button_text', __( 'Take This Course', 'learndash-stripe' ) );
	    }
	}

	$checkout = '<div class="learndash_checkout_button learndash_2checkout_button">
		<form action="' . esc_url( $action_url ) . '" method="post">
			<input type="hidden" name="sid" value="' . esc_attr( $sid ) . '" >
			<input type="hidden" name="mode" value="2CO" >
			<input type="hidden" name="li_0_type" value="product" >
			<input type="hidden" name="li_0_name" value="' . esc_attr( $title ) . '" >
			' . $recurrence . '
			<input type="hidden" name="li_0_product_id" value="' . esc_attr( $productid ) . '" >
			<input type="hidden" name="li_0_price" value="' . esc_attr( $price ) . '" >
			<input type="hidden" name="li_0_quantity" value="1" >
			<input type="hidden" name="li_0_tangible" value="N" >
			<input type="hidden" name="demo" value="' . esc_attr( $demo ) . '" >
			<input type="hidden" name="merchant_order_id" value="Order ID: ' . esc_attr( $order_id ) . '" >
			<input type="hidden" name="user_id" value="' . esc_attr( $user_id ) . '" >
			<input type="hidden" name="course_id" value="' . esc_attr( $productid ) . '" >
			<input type="hidden" name="x_receipt_link_url" value="' . esc_attr( $x_receipt_link_url ) . '" >
			<input type="submit" class="learndash-2checkout-checkout-button btn-join button" value="' . esc_attr( $button_text ) . '"/>
		</form>
	</div>';

	return $button . $checkout;
}

function learndash_2checkout_output_message()
{
	if ( empty( $_GET['learndash-2co-message'] ) ) {
		return;
	}

	$type = sanitize_text_field( $_GET['learndash-2co-message'] );

	$message = '';
	switch ( $type ) {
		case 'success':
			$message = __( 'Your transaction is successful. Please check your email for the login credentials.', 'learndash-2checkout' );
			break;
		
		case 'invalid':
			$message = __( 'Your transaction is invalid. Please contact our web administrator to resolve this issue.', 'learndash-2checkout' );
			break;
	}

	$message = apply_filters( 'learndash_2checkout_message', $message, $type );

	if ( empty( $message ) ) {
		return;
	}
	?>
	<script type="text/javascript">
		jQuery( document ).ready( function( $ ) {
			alert( '<?php echo esc_attr( $message ) ?>' );	
		} );
	</script>
	<?php
}